#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['brutalmaze', 'brutalmaze.soundfx']

package_data = \
{'': ['*']}

install_requires = \
['loca', 'palace', 'pygame>=2']

entry_points = \
{'console_scripts': ['brutalmaze = brutalmaze.__main__:main']}

setup(name='brutalmaze',
      version='1.1.1',
      description="Minimalist thrilling shoot 'em up game",
      author='Nguyễn Gia Phong',
      author_email='mcsinyx@disroot.org',
      url='https://sr.ht/~cnx/brutalmaze',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.7',
     )
